// timer_1A auf timer_2 gendert, Anzeige gendert  24.1.2014   P.Winges


#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include "portbits.h"
#include "8x12_horizontal_MSB_1.h"
#include "global.h"
#include "lcd.h"

unsigned char cgram[16][8];
unsigned char viewstart;

unsigned char GrayCnt=3;

register unsigned char R_RAS asm("r3");
register unsigned char R_CAS asm("r4");
register unsigned char R_RAS_W asm("r5");
register unsigned char R_CAS_W asm("r6");

void lcd_init(void)
{	cli();
	wdt_disable();

	R_RAS = C_RAS;
	R_CAS = C_CAS;
	R_RAS_W = C_RAS_W;
	R_CAS_W = C_CAS_W;

	lcd_clear();

	lcd_block(10,10,309,80,0,255);

	lcd_block(16,16,74,74,255,0);
	lcd_block(24,24,66,66,128,64);
	lcd_block(32,32,58,58,64,128);
	lcd_block(40,40,50,50,0,255);
	
	lcd_block(76,16,134,74,255,0);
	lcd_circle(105,45,24,0,64);
	lcd_circle(105,45,18,64,128);
	lcd_circle(105,45,12,128,255);
	lcd_circle(105,45,6,255,64);
	
	lcd_block(137,17,195,73,255,255);
	lcd_block(137,31,195,45,128,255);
	lcd_block(137,45,195,59,64,255);
	lcd_block(137,59,195,73,0,255);
	
	lcd_block(200,17,256,73,255,255);
	lcd_block(214,17,228,73,128,255);
	lcd_block(228,17,242,73,64,255);
	lcd_block(242,17,256,73,0,255);

		
	lcd_string(6,100,PSTR("320x240 LCD Controller + DRam"),255,0);	           //Anzeige gendert  P.Winges
#ifdef GRAYSCALE
	//lcd_string(6,112,PSTR("4 Graustufen"),255,0);
	lcd_string(6,112,PSTR("4 Graustufen \xB8"" by NRG""   28800 baud"),255,0);   //Anzeige gendert  P.Winges
#endif	
	//lcd_string(6,120,PSTR("\xB8"" by NRG"),255,0);
	// THX an Benedikt
    
	//OCR2A=RELOAD;              //fr Atmaga_88
	//TCCR2A=(1<<WGM21);							
	//TCCR2B=2;
	//TIMSK2=(1<<OCIE2A);
                                  //fr Atmega8
      OCR2=RELOAD;                // timer_1A auf timer_2 gendert  P.Winges
	TCCR2=(1<<WGM21)|2;         // 2MHz Timer Takt, CTC
	TIMSK=(1<<OCIE2);
	
	sei();

	wdt_enable(WDTO_250MS);
	ENABLE_VLCD=1;
}

void lcd_string(unsigned char x, unsigned char y, char *txt, unsigned char tcol, unsigned char bcol)
{	unsigned char c;
	for(;;)
	{	c=pgm_read_byte(txt++);
		if (!c)
			break;
		lcd_writechar(x++,y,c,tcol,bcol);
	}
}


void bresenham_line (unsigned short x1, unsigned short y1, unsigned short x2, unsigned short y2, unsigned char color)
{	
	signed short dx,dy,x,y;
	signed char sdx,sdy;
	unsigned short i,px,py;
	dx=x2-x1;      // the horizontal distance of the line
	dy=y2-y1;      // the vertical distance of the line
	if (dx==0)
		sdx=0;
	if (dx>0)
		sdx=1;
	else
	{  	sdx=-1;
		dx=-dx;
	}
	if (dy==0)
		sdy=0;
	if (dy>0)
		sdy=1;
	else
	{	sdy=-1;
		dy=-dy;
	}
	x=dx*2;
	y=dy*2;
	px=x1;
	py=y1;
	lcd_setpixel (px,py,color);

	if (dx>=dy)					 // the line is more horizontal than vertical
	{	for(i=0;i<dx;i++)
		{	y+=dy;
			if (y>=dx)
			{	y-=dx;
				py+=sdy;
			}
			px+=sdx;
			lcd_setpixel (px,py,color);
		}
	}
	else				 		// the line is more vertical than horizontal
	{	for(i=0;i<dy;i++)
		{	x+=dx;
			if (x>=dy)
			{	x-=dy;
				px+=sdx;
			}
			py+=sdy;
			lcd_setpixel (px,py,color);
		}
	}
	
}

void lcd_line (unsigned short x1, unsigned short y1, unsigned short x2, unsigned short y2, unsigned char color)
{   
	unsigned short temp;
	if (y1==y2)
	{	if (x1>x2)
		{	temp=x1;
			x1=x2;
			x2=temp;
		}
		for (temp=x1; temp<=x2; temp++){
			if ((temp & 7)==0 && (x2-temp)>7){
#ifdef GRAYSCALE
				lcd_writebyteex(temp/8,y1,255,color,0);
#else
				if(color & 0b01000000)
					lcd_writebyte(temp/8,y1,255);
				else
					lcd_writebyte(temp/8,y1,0);
#endif				
				temp += 7;
			}
			else
				lcd_setpixel (temp,y1,color);
		}
	}
	else if (x1==x2)
	{	if (y1>y2)
		{	temp=y1;
			y1=y2;
			y2=temp;
		}
		for (temp=y1; temp<=y2; temp++)
			lcd_setpixel (x1,temp,color);
	}
	else
		bresenham_line (x1, y1, x2, y2, color);
}

void lcd_circle(unsigned short xm, unsigned short ym, unsigned char r, unsigned char color1, unsigned char color2)
{	
	unsigned short x;
	unsigned char y=r;
	signed short p;
	if ((r>xm)||(r>ym)||(r<1))
		return;
	y=r;
	p=(3-(2*r));
	for (x=0; x<=y; x++)
	{	lcd_line(xm-x,ym+y,xm+x,ym+y,color1);
		lcd_line(xm-x,ym-y,xm+x,ym-y,color1);
		lcd_line(xm-y,ym+x,xm+y,ym+x,color1);
		lcd_line(xm-y,ym-x,xm+y,ym-x,color1);
		if(p<0)
			p+=((4*x)+6);
		else
		{	y--;
			p+=((4*(x-y))+10);
		}
	}
	y=r;
	p=(3-(2*r));
	for (x=0; x<=y; x++)
	{	lcd_setpixel((xm+x),(ym+y),color2);
		lcd_setpixel((xm+y),(ym+x),color2);
		lcd_setpixel((xm+y),(ym-x),color2);
		lcd_setpixel((xm+x),(ym-y),color2);
		lcd_setpixel((xm-x),(ym-y),color2);
		lcd_setpixel((xm-y),(ym-x),color2);
		lcd_setpixel((xm-y),(ym+x),color2);
		lcd_setpixel((xm-x),(ym+y),color2);
		if(p<0)
			p+=((4*x)+6);
		else
		{	y--;
			p+=((4*(x-y))+10);
		}
	}

}

void lcd_block (unsigned short x1, unsigned short y1, unsigned short x2, unsigned short y2, unsigned char color1, unsigned char color2)
{	
	unsigned short y;
	if (x2>x1+1)
	{	for (y=y1+1; y<y2; y++)
			lcd_line(x1+1,y,x2-1,y,color1);
	}
	lcd_line(x1,y1,x2,y1,color2);
	lcd_line(x2,y1,x2,y2,color2);
	lcd_line(x2,y2,x1,y2,color2);
	lcd_line(x1,y2,x1,y1,color2);
	
}

void lcd_writecgram(unsigned char x, unsigned char y, unsigned char nr,unsigned char textcol,unsigned char backcol)
{	
	nr&=15;
	lcd_writebyteex(x,y+0,cgram[nr][0],textcol, backcol);
	lcd_writebyteex(x,y+1,cgram[nr][0],textcol, backcol);
	lcd_writebyteex(x,y+2,cgram[nr][1],textcol, backcol);
	lcd_writebyteex(x,y+3,cgram[nr][2],textcol, backcol);
	lcd_writebyteex(x,y+4,cgram[nr][2],textcol, backcol);
	lcd_writebyteex(x,y+5,cgram[nr][3],textcol, backcol);
	lcd_writebyteex(x,y+6,cgram[nr][4],textcol, backcol);
	lcd_writebyteex(x,y+7,cgram[nr][5],textcol, backcol);
	lcd_writebyteex(x,y+8,cgram[nr][5],textcol, backcol);
	lcd_writebyteex(x,y+9,cgram[nr][6],textcol, backcol);
	lcd_writebyteex(x,y+10,cgram[nr][7],textcol, backcol);
	lcd_writebyteex(x,y+11,cgram[nr][7],textcol, backcol);
	
}

void lcd_setstartx(unsigned char pos)
{
	if (pos>(XVSIZESW-XSIZE)/8-1)
		pos=(XVSIZESW-XSIZE)/8-1;
	pos*=4;
	viewstart=pos;
}
